@@ -9,10 +9,10 @@ import tempfile
 import shutil
 
 prefix = '@prefix@'
-exec_prefix = '@exec_prefix@'
+bindir = '@bindir@'
 datadir = '@datadir@'
 libdir = '@libdir@'
-
+exec_prefix = '@exec_prefix@'
 
 def interpolate (str):
 	str = string.replace (str, '{', '(')
@@ -22,6 +22,7 @@ def interpolate (str):
 
 if prefix != '@' + 'prefix@':
 	exec_prefix = interpolate (exec_prefix) % vars ()
+	bindir = interpolate (bindir) % vars ()
 	datadir = os.path.join (interpolate (datadir) % vars (), 'mftrace')
 	libdir = interpolate (libdir) % vars ()
 
@@ -194,7 +195,7 @@ def help ():
 		('\n'),
 		(options_help_str (option_definitions)),
 		('\n\n'),
-		(_ ("Report bugs to %s") % 'hanwen@cs.uu.nl'),
+		(_ ("Report bugs to %s") % 'hanwen@xs4all.nl'),
 		('\n')]
 	map (sys.stdout.write, ls)
 
@@ -269,7 +270,6 @@ Example:
 option_definitions = [
 	('', 'h', 'help', _ ("This help")),
 	('', 'k', 'keep', _ ("Keep all output in directory %s.dir") % program_name),
-	('', '', 'keep-all', _ ("Really keep all output (implies --keep)")),
 	('MAG', '', 'magnification', _ ("Set magnification for MF to MAG (default: 1000)")),
 	('', 'V', 'verbose', _ ("Verbose")),
 	('', 'v', 'version', _ ("Print version number")),
@@ -280,7 +280,6 @@ option_definitions = [
 	('LIST', '', 'glyphs', _ ('Process only these glyphs.  LIST is comma separated')),
 	('FILE', '', 'tfmfile', _ ("Use FILE for the TFM file")),
 	('FILE', 'e', 'encoding', _ ("Use encoding file FILE")),
-	('', 't', 'truetype', _ ("Generate TrueType file (requires fontforge)")),
 	('', '', 'keep-trying', _ ("Don't stop if tracing fails")),
 	('', 'w', 'warranty', _ ("show warranty and copyright")),
 	('', '', 'dos-kpath', _ ("try to use Miktex kpsewhich")),
@@ -381,10 +380,9 @@ def make_pbm (filename, outname, char_number):
 	""" Extract bitmap from the PK file FILENAME (absolute) using `gf2pbm'.
 	Return FALSE if the glyph is not valid.
 	"""
-
-	command = "gf2pbm -n %d -o %s %s" % (char_number, outname, filename)
+	
+	command = "%s/gf2pbm -n %d -o %s %s" % (bindir, char_number, outname, filename)
 	status = system (command, ignore_error = 1)
-
 	return (status == 0)
 
 def read_encoding (file):
@@ -596,7 +594,7 @@ def potrace_path_to_type1_ops (at_file, bitmap_metrics, tfm_wid):
 	return (bbox, t1_outline)
 
 def read_gf_dims (name, c):
-	str = popen ('gf2pbm -n %d -s %s' % (c, name)).read ()
+	str = popen ('%s/gf2pbm -n %d -s %s' % (bindir, c, name)).read ()
 	m = re.search ('size: ([0-9]+)+x([0-9]+), offset: \(([0-9-]+),([0-9-]+)\)', str)
 
 	return tuple (map (string.atoi, m.groups ()))
@@ -612,13 +610,20 @@ def trace_font (fontname, gf_file, metric, glyphs, encoding,
 
 	# for single glyph testing.
 	# glyphs = []
+	first_p = 1
+	global verbose_p
+	vp = verbose_p
 	for a in glyphs:
+    		if encoding[a] == ".notavail":
+        		continue
 		valid = metric.has_char (a)
 		if not valid:
+        		encoding[a] = ".notavail"
 			continue
 
 		valid = make_pbm (gf_file, 'char.pbm', a)
 		if not valid:
+        		encoding[a] = ".notavail"
 			continue
 
 		(w, h, xo, yo) = read_gf_dims (gf_file, a)
@@ -632,6 +637,7 @@ def trace_font (fontname, gf_file, metric, glyphs, encoding,
 		if not success:
 			sys.stderr.write ("(skipping character)]")
 			sys.stderr.flush ()
+        		encoding[a] = ".notavail"
 			continue
 
 		if not verbose_p:
@@ -643,12 +649,17 @@ def trace_font (fontname, gf_file, metric, glyphs, encoding,
 						 tw)
 
 		if t1o == '':
+        		encoding[a] = ".notavail"
 			continue
 
 		font_bbox = update_bbox_with_bbox (font_bbox, bbox)
 
 		t1os.append ('/%s %s ' % (encoding[a], t1o))
 
+		if first_p:
+			verbose_p = 0
+
+	verbose_p = vp
 	progress ('\n')
 	to_type1 (t1os, font_bbox, fontname, encoding, magnification, fontinfo )
 
@@ -657,7 +668,8 @@ def ps_encode_encoding (encoding):
 	      % (len (encoding), len (encoding)-1)
 
 	for i in range (0, len (encoding)):
-		str = str + 'dup %d /%s put\n' % (i, encoding[i])
+    		if encoding[i] != ".notavail":
+			str = str + 'dup %d /%s put\n' % (i, encoding[i])
 
 	return str
 
@@ -727,7 +739,7 @@ cleartomark
 	vars = {
 		'VVV': '001',
 		'WWW': '001',
-		'Notice': 'Generated from MetaFont bitmap by mftrace %s, http://www.cs.uu.nl/~hanwen/mftrace/ ' % program_version,
+		'Notice': 'Generated from MetaFont bitmap by mftrace %s, http://www.xs4all.nl/~hanwen/mftrace/ ' % program_version,
 		'UnderlinePosition': '-100',
 		'UnderlineThickness': '50',
 		'xrevscale': 1.0/1000.0,
@@ -751,12 +763,6 @@ cleartomark
 
 	opt = ''
 
-	outname = fontname + '.pfa'
-
-	progress (_ ("Assembling raw font to `%s'... ") % outname)
-	system ('t1asm --pfa mftrace.t1asm %s' % outname + '.raw')
-	progress ('\n')
-
 def update_bbox_with_point (bbox, pt):
 	(llx, lly, urx, ury) = bbox
 	llx = min (pt[0], llx)
@@ -804,44 +810,146 @@ def get_fontforge_command ():
 		return ''
 	return fontforge_cmd
 
-def make_outputs (fontname, formats):
-	"""
-	run pfaedit to convert to other formats
-	"""
+def tfm2kpx (tfmname, encoding):
+	kpx_lines = []
+	pl = popen ("tftopl %s" % (tfmname))
+	
+	label_pattern = re.compile (
+		"\A   \(LABEL ([DOHC]{1}) ([A-Za-z0-9]*)\)")
+	krn_pattern = re.compile (
+		"\A   \(KRN ([DOHC]{1}) ([A-Za-z0-9]*) R (-?[\d\.]+)\)")
 
-	ff_command = get_fontforge_command ()
-	if not ff_command:
-		shutil.copy2 (fontname + '.pfa.raw',
-			      fontname + '.pfa')
-		return 0
+	first = 0
+	second = 0
 
-	# not used?
-	if round_to_int :
-		round_cmd = 'RoundToInt();\n'
+	for line in pl.readlines ():
+		
+		label_match = label_pattern.search (line)
+		if not (label_match is None):
+			if label_match.group (1) == "D":
+				first = int (label_match.group (2))
+			elif label_match.group (1) == "O":
+				first = string.atoi (label_match.group (2), 8)
+			elif label_match.group (1) == "C":
+				first = ord (label_match.group (2))
+			
+		krn_match = krn_pattern.search (line)
+		if not (krn_match is None):
+			if krn_match.group (1) == "D":
+				second = int (krn_match.group (2))
+			elif krn_match.group (1) == "O":
+				second = string.atoi (krn_match.group (2), 8)
+			elif krn_match.group (1) == "C":
+				second = ord (krn_match.group (2))
+			
+			krn = round (float (krn_match.group (3)) * 1000)
+			
+			if (encoding[first] != '.notavail' and 
+				encoding[first] != '.notdef' and
+				encoding[second] != '.notavail' and 
+				encoding[second] != '.notdef'):
+				
+				kpx_lines.append ("KPX %s %s %d\n" % (
+					encoding[first], encoding[second], krn))
+	
+	return kpx_lines
 
-	generate_cmds = ''
-	for f in formats:
-		generate_cmds += 'Generate("%s");' % (filename  + '.' + f)
+def get_afm (t1_path, tfmname, encoding, out_path):
+	afm_stream = popen ("printafm %s" % (t1_path))
+	afm_lines = []
+	kpx_lines = tfm2kpx (tfmname, encoding)
+	
+	for line in afm_stream.readlines ():
+		afm_lines.append (line)
+		
+		if re.match (r"^EndCharMetrics", line, re.I):
+			afm_lines.append ("StartKernData\n")
+			afm_lines.append ("StartKernPairs %d\n" % len (kpx_lines))
+			
+			for kpx_line in kpx_lines:
+				afm_lines.append (kpx_line)
+			
+			afm_lines.append ("EndKernPairs\n")
+			afm_lines.append ("EndKernData\n")
+	 
+	progress (_ ("Writing metrics to `%s'... ") % out_path)
+	afm_file = open (out_path, 'w')
+	afm_file.writelines (afm_lines)
+	afm_file.flush ()
+	afm_file.close ()
+	
+	progress ('\n')
+
+def assemble_font (fontname, format, is_raw):
+    ext = '.' + format
+    asm_opt = '--pfa'
+    
+    if format == 'pfb':
+        asm_opt = '--pfb'
+
+    if is_raw:
+        ext = ext + '.raw'
+    
+    outname = fontname + ext
+
+    progress (_ ("Assembling raw font to `%s'... ") % outname)
+    system ('t1asm %s mftrace.t1asm %s' % (asm_opt, outname))
+    progress ('\n')
+    return outname
+
+def make_outputs (fontname, formats, encoding):
+  	"""
+  	run pfaedit to convert to other formats
+  	"""
+  
+	ff_needed = 0
+	ff_command = ""
+    
+	if (simplify_p or round_to_int or 'ttf' in formats or 'svg' in formats):
+		ff_needed = 1
+	if ff_needed:
+		ff_command = get_fontforge_command ()
+    
+	if ff_needed and ff_command:
+		raw_name = assemble_font (fontname, 'pfa', 1)
+
+		simplify_cmd = ''
+		if round_to_int:
+			simplify_cmd = 'RoundToInt ();'
+		generate_cmds = ''
+		for f in formats:
+			generate_cmds += 'Generate("%s");' % (filename  + '.' + f)
+
+		if simplify_p:
+			simplify_cmd ='''SelectAll ();
 
-	simplify_cmd = ''
-	if simplify_p:
-		simplify_cmd ='''SelectAll ();
-SelectAll ();
 AddExtrema();
 Simplify ();
-AutoHint ();'''
+%(simplify_cmd)s
+AutoHint ();''' % vars()
 
-	open ('to-ttf.pe', 'w').write ('''#!/usr/bin/env %(ff_command)s
+		open ('to-ttf.pe', 'w').write ('''#!/usr/bin/env %(ff_command)s
 Open ($1);
 MergeKern($2);
-%(round_cmd)s
 %(simplify_cmd)s
 %(generate_cmds)s
 Quit (0);
 ''' % vars())
 
-	system ("%s -script to-ttf.pe %s %s" % (ff_command,
-						(filename + '.pfa.raw'), tfmfile))
+		system ("%s -script to-ttf.pe %s %s" % (ff_command,
+			    (raw_name), tfmfile))
+	else:
+		t1_path = ''
+	
+		if ('pfa' in formats):
+			t1_path = assemble_font (fontname, 'pfa', 0)
+
+		if ('pfb' in formats):
+			t1_path = assemble_font (fontname, 'pfb', 0)
+	
+		if (t1_path != '' and 'afm' in formats):
+			get_afm (t1_path, tfmfile, encoding, fontname + '.afm')
+
 
 def getenv (var, default):
 	if os.environ.has_key (var):
@@ -903,8 +1011,11 @@ def gen_pixel_font (filename, metric, magnification):
 
 		logfile = '%s.log' % filename
 		log = ''
+                prod = 0
 		if os.path.exists (logfile):
 			log = open (logfile).read ()
+			m = re.search ('Output written on %s.([0-9]+)gf' % re.escape (filename), log)
+			prod = string.atoi (m.group (1))
 
 		if st:
 			sys.stderr.write ('\n\nMetafont failed.  Excerpt from the log file: \n\n*****')
@@ -920,9 +1031,12 @@ Apparently, some numbers overflowed.  Try using --magnification with a
 lower number.  (Current magnification: %d)
 """ % magnification)
 
-			sys.exit (1)
-		m = re.search ('Output written on %s.([0-9]+)gf' % re.escape (filename), log)
-		prod = string.atoi (m.group (1))
+			if not keep_trying_p or prod == 0:
+				sys.exit (1)
+			else:
+				sys.stderr.write ('\n\nTrying to proceed despite of the Metafont errors...\n')
+		
+	    
 
 	return "%s.%d" % (filename, prod)
 
@@ -1127,6 +1241,8 @@ output_name = ''
 gf_fontname = ''
 encoding_file_override = ''
 glyph_range = []
+glyph_ranges = []
+glyph_subrange = []
 for (o, a) in options:
 	if 0:
 		pass
@@ -1150,11 +1266,15 @@ for (o, a) in options:
 	elif o == '--gffile':
 		gf_fontname = a
 	elif o == '--glyphs':
-		glyph_range = map (string.atoi, string.split (a, ','))
+		glyph_ranges = string.split (a, ',')
+		for r in glyph_ranges:
+			glyph_subrange = map (string.atoi, string.split (r, '-'))
+			if len (glyph_subrange) == 2 and glyph_subrange[0] < glyph_subrange[1] + 1:
+				glyph_range.extend (range (glyph_subrange[0], glyph_subrange[1] + 1))
+			else:
+				glyph_range.append (glyph_subrange[0])
 	elif o == '--tfmfile':
 		tfmfile = a
-	elif o == '--truetype' or o == '-t':
-		ttf_p = 1
 	elif o == '--dos-kpath':
 		dos_kpath_p = 1
 	elif o == '--formats' or o == '-f':
@@ -1194,7 +1314,7 @@ if trace_binary != 'potrace' and stat == 0:
 	path_to_type1_ops = autotrace_path_to_type1_ops
 
 if not trace_binary:
-	error (_ ("No tracing program found. Exit."))
+	error (_ ("No tracing program found.\nInstall potrace or autotrace."))
 
 
 identify (sys.stderr)
@@ -1272,7 +1392,7 @@ for filename in files:
 	trace_font (basename, gf_fontname, metric, glyph_range, encoding,
 		    magnification, fontinfo)
 		
-	make_outputs (basename, formats)
+	make_outputs (basename, formats, encoding)
 	for format in formats:
 		shutil.copy2 (basename + '.' + format, origdir)
 
